/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EJB3DeploymentDescLoader.java 4510 2009-02-03 16:19:39Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.xml.parsing;

import static org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback.POST_ACTIVATE;
import static org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback.PRE_PASSIVATE;

import java.net.URL;
import java.util.List;

import javax.ejb.TransactionAttributeType;

import org.ow2.util.ee.metadata.common.api.xml.struct.IActivationConfig;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvironment;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.CommonDeploymentDescLoader;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.common.impl.xml.struct.ActivationConfig;
import org.ow2.util.ee.metadata.common.impl.xml.struct.AroundInvoke;
import org.ow2.util.ee.metadata.common.impl.xml.struct.MethodDD;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IAssemblyDescriptor;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IContainerTransaction;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IEnterpriseBeans;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IInterceptorBinding;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IInterceptors;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IMessageDestination;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IMethodPermission;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IRemoveMethod;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.ISession;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JApplicationException;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.AbsBean;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.AssemblyDescriptor;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.ContainerTransaction;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EJB3;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EnterpriseBeans;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Interceptor;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.InterceptorBinding;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Interceptors;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.MessageDestination;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.MessageDriven;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.MethodPermission;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.RemoveMethod;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Session;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.xml.DocumentParser;
import org.ow2.util.xml.DocumentParserException;
import org.ow2.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * This class analyzes the EJB3 deployment descriptor and fill a struct with
 * getters.
 * @author Florent Benoit
 */
public final class EJB3DeploymentDescLoader extends CommonDeploymentDescLoader {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(EJB3DeploymentDescLoader.class);

    /**
     * EJB3 instance object that is the result of the parsing.
     */
    private EJB3 ejb3 = null;

    /**
     * Utility class, no public constructor.
     * @param url the url that is pointing to the EJB Xml file.
     */
    private EJB3DeploymentDescLoader(final URL url) {
        super(url);
        this.ejb3 = new EJB3();
    }

    /**
     * Load the ejb-jar.xml file.
     * @param url the URL of the XML file.
     * @throws ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    public static EJB3 loadDeploymentDescriptor(final URL url) throws ParsingException {
        logger.debug("Analyzing url {0}", url);

        EJB3DeploymentDescLoader ejbDeploymentDescLoader = new EJB3DeploymentDescLoader(url);
        ejbDeploymentDescLoader.parse();
        return ejbDeploymentDescLoader.getEJB3();
    }

    /**
     * Analyze the URL and build an EJB3 object which contains data of the XML.
     * @throws ParsingException if the analyze of the file fails.
     */
    public void parse() throws ParsingException {
        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(getURL(), isValidating(), new EJB3EntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <ejb-jar>
        Element ejbJarRootElement = document.getDocumentElement();

        // Define NS
        defineXMLNS(ejbJarRootElement);

        // enterprise-beans
        NodeList enterprisebeansList = ejbJarRootElement.getElementsByTagNameNS(getXMLNS(), IEnterpriseBeans.NAME);
        if (enterprisebeansList.getLength() == 1) {
            Element enterpriseBeansElement = (Element) enterprisebeansList.item(0);

            // Create struct
            IEnterpriseBeans enterpriseBeans = new EnterpriseBeans();
            ejb3.setEnterpriseBeans(enterpriseBeans);

            // Analyze beans
            analyzeBeans(enterpriseBeansElement, enterpriseBeans);

        }

        // interceptors
        NodeList interceptorsList = ejbJarRootElement.getElementsByTagNameNS(getXMLNS(), IInterceptors.NAME);
        if (interceptorsList.getLength() == 1) {
            Element interceptorsElement = (Element) interceptorsList.item(0);

            // Create struct
            IInterceptors interceptors = new Interceptors();
            ejb3.setInterceptors(interceptors);

            // Analyze the interceptors element
            analyzeInterceptorsElement(interceptorsElement, interceptors);
        }

        // assembly-descriptor
        NodeList assemblyDescriptorList = ejbJarRootElement.getElementsByTagNameNS(getXMLNS(), IAssemblyDescriptor.NAME);
        if (assemblyDescriptorList.getLength() == 1) {
            Element assemblyDescriptorElement = (Element) assemblyDescriptorList.item(0);

            // Create struct
            IAssemblyDescriptor assemblyDescriptor = new AssemblyDescriptor();
            ejb3.setAssemblyDescriptor(assemblyDescriptor);

            // Analyze the assembly-descriptor element
            analyzeAssemblyDescriptor(assemblyDescriptorElement, assemblyDescriptor);
        }

        if ("true".equals(ejbJarRootElement.getAttribute("metadata-complete"))) {
            ejb3.setMetadataComplete(true);
        } else {
            ejb3.setMetadataComplete(false);
        }

    }

    /**
     * Analyze common stuff of beans.
     * @param enterpriseBeansElement the dom element.
     * @param enterpriseBeans the structure representing object.
     */
    private void analyzeBeans(final Element enterpriseBeansElement, final IEnterpriseBeans enterpriseBeans) {
        // Get session beans
        NodeList sessionBeansList = enterpriseBeansElement.getElementsByTagNameNS(getXMLNS(), ISession.NAME);

        // Loop on session beans
        for (int i = 0; i < sessionBeansList.getLength(); i++) {
            Element sessionElement = (Element) sessionBeansList.item(i);

            // Build instance that is created.
            Session session = new Session();
            enterpriseBeans.addSession(session);

            // Analyze session bean
            analyzeSessionBean(sessionElement, session);
        }

        // Get MDB beans
        NodeList messageDrivenBeansList = enterpriseBeansElement.getElementsByTagNameNS(getXMLNS(), IMessageDriven.NAME);

        // Loop on MDB
        for (int i = 0; i < messageDrivenBeansList.getLength(); i++) {
            Element mdbElement = (Element) messageDrivenBeansList.item(i);

            // Build instance that is created.
            MessageDriven messageDriven = new MessageDriven();
            enterpriseBeans.addMessageDriven(messageDriven);

            // Analyze MDB
            analyzeMessageDrivenBean(mdbElement, messageDriven);
        }

    }

    /**
     * Analyze common stuff of a bean.
     * @param beanElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeBean(final Element beanElement, final AbsBean bean) {
        // Get ejb-name
        String ejbName = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "ejb-name");
        bean.setEjbName(ejbName);

        // Get ejb-class
        String ejbClass = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "ejb-class");
        bean.setEjbClass(ejbClass);

        // Security identity (run-as) if any
        String runAsRole = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "run-as");
        if (runAsRole != null) {
            bean.setRunAsRole(runAsRole);
        }

        // Mapped Name
        String mappedName = XMLUtils.getChildStringValueForElement(beanElement, "mapped-name");
        if (mappedName != null) {
            bean.setMappedName(mappedName);
        }

        // Get transaction-type
        String transactionType = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "transaction-type");
        bean.setTransactionType(transactionType);

        // Analyze common things
        analyzeEnvironment(beanElement, bean);
    }

    /**
     * Analyze XML of a MDB.
     * @param beanElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeMessageDrivenBean(final Element beanElement, final MessageDriven bean) {
        analyzeBean(beanElement, bean);

        // Message destination type
        String messageDestinationType = XMLUtils.getChildStringValueForElement(beanElement, "message-destination-type");
        bean.setMessageDestinationType(messageDestinationType);

        // Message destination link
        String messageDestinationLink = XMLUtils.getChildStringValueForElement(beanElement, "message-destination-link");
        bean.setMessageDestinationLink(messageDestinationLink);

        // Messaging type
        String messagingType = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "messaging-type");
        bean.setMessagingType(messagingType);

        // Get activation-config
        NodeList activationConfigList = beanElement.getElementsByTagNameNS(getXMLNS(), IActivationConfig.NAME);
        // Loop on the activation-config
        for (int i = 0; i < activationConfigList.getLength(); i++) {
            Element activationConfigElement = (Element) activationConfigList.item(i);

            // Get activation-config-property
            NodeList activationConfigPropertyList = activationConfigElement.getElementsByTagNameNS(getXMLNS(),
                    "activation-config-property");
            for (int j = 0; j < activationConfigPropertyList.getLength(); j++) {
                Element activationConfigPropertyElement = (Element) activationConfigPropertyList.item(j);

                // Build Object
                ActivationConfig activationConfig = new ActivationConfig();
                bean.addActivationConfig(activationConfig);
                String name = XMLUtils.getStringValueElement(getXMLNS(), activationConfigPropertyElement,
                        "activation-config-property-name");
                String value = XMLUtils.getStringValueElement(getXMLNS(), activationConfigPropertyElement,
                        "activation-config-property-value");
                activationConfig.setName(name);
                activationConfig.setValue(value);
            }
        }

    }

    /**
     * Analyze XML of a session bean.
     * @param beanElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeSessionBean(final Element beanElement, final Session bean) {
        analyzeBean(beanElement, bean);

        // Get business-remote
        List<String> businessRemoteList = XMLUtils.getStringListValueElement(getXMLNS(), beanElement, "business-remote");
        for (String businessRemote : businessRemoteList) {
            bean.addBusinessRemote(businessRemote);
        }

        // Get business-local
        List<String> businessLocalList = XMLUtils.getStringListValueElement(getXMLNS(), beanElement, "business-local");
        for (String businessLocal : businessLocalList) {
            bean.addBusinessLocal(businessLocal);
        }

        // Remove Methods
        NodeList removeMethodList = beanElement.getElementsByTagNameNS(getXMLNS(), IRemoveMethod.NAME);

        // Loop on the remove-method
        for (int i = 0; i < removeMethodList.getLength(); i++) {
            Element removeMethodElement = (Element) removeMethodList.item(i);

            RemoveMethod removeMethod = new RemoveMethod();
            bean.addRemoveMethod(removeMethod);

            // Analyze method
            NodeList beanMethodList = removeMethodElement.getElementsByTagNameNS(getXMLNS(), "bean-method");
            removeMethod.setMethod(getMethodFromElement((Element) beanMethodList.item(0)));

            // Retain if exception
            String retainIfExceptionStr = XMLUtils.getStringValueElement(getXMLNS(), removeMethodElement, "retain-if-exception");
            if (retainIfExceptionStr != null) {
                // convert into boolean value
                boolean retainIfException = false;
                retainIfException = Boolean.parseBoolean(retainIfExceptionStr);
                removeMethod.setRetainIfException(retainIfException);
            }
        }

        // Get remote home
        String remoteHome = XMLUtils.getChildStringValueForElement(beanElement, "home");
        bean.setRemoteHome(remoteHome);

        // Get Local home
        String localHome = XMLUtils.getChildStringValueForElement(beanElement, "local-home");
        bean.setLocalHome(localHome);

        // Get session-type
        String sessionType = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "session-type");
        bean.setSessionType(sessionType);

    }

    /**
     * Gets the methodDD from the given dom element.
     * @param methodElement the dom element to analyze
     * @return the given method object
     */
    private MethodDD getMethodFromElement(final Element methodElement) {
        // create object
        MethodDD method = new MethodDD();

        // Analyze name
        String methodName = XMLUtils.getStringValueElement(getXMLNS(), methodElement, "method-name");
        method.setName(methodName);

        // Sometime there is an ejb-name
        String ejbName = XMLUtils.getStringValueElement(getXMLNS(), methodElement, "ejb-name");
        method.setEjbName(ejbName);

        // Params
        NodeList methodParamsList = methodElement.getElementsByTagNameNS(getXMLNS(), "method-params");
        if (methodParamsList != null && methodParamsList.getLength() == 1) {
            // first item is the method-params element
            Element methodParamsElement = (Element) methodParamsList.item(0);

            List<String> paramList = XMLUtils.getStringListValueElement(getXMLNS(), methodParamsElement, "method-param");
            for (String param : paramList) {
                method.addParam(param);
            }

        }
        return method;
    }

    /**
     * Analyze the interceptors element of an EJB-JAR.
     * @param interceptorsElement the dom element.
     * @param interceptors the structure representing object.
     */
    private void analyzeInterceptorsElement(final Element interceptorsElement, final IInterceptors interceptors) {
        // Get interceptor list elements
        NodeList interceptorList = interceptorsElement.getElementsByTagNameNS(getXMLNS(), IInterceptor.NAME);

        // Loop on <interceptor> elements
        for (int i = 0; i < interceptorList.getLength(); i++) {
            Element interceptorElement = (Element) interceptorList.item(i);

            Interceptor interceptor = new Interceptor();
            interceptors.addInterceptor(interceptor);

            // Extract interceptor class
            String interceptorClass = XMLUtils.getStringValueElement(getXMLNS(), interceptorElement, "interceptor-class");
            interceptor.setInterceptorClass(interceptorClass);

            // Analyze the object
            analyzeEnvironment(interceptorElement, interceptor);
        }
    }

    /**
     * Analyze the assembly descriptor of an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeAssemblyDescriptor(final Element assemblyDescriptorElement,
            final IAssemblyDescriptor assemblyDescriptor) {
        analyzeApplicationException(assemblyDescriptorElement, assemblyDescriptor);

        analyzeInterceptorBinding(assemblyDescriptorElement, assemblyDescriptor);

        analyzeContainerTransaction(assemblyDescriptorElement, assemblyDescriptor);

        analyzeSecurity(assemblyDescriptorElement, assemblyDescriptor);

        analyzeMessageDestination(assemblyDescriptorElement, assemblyDescriptor);
    }

    /**
     * Analyze the interceptor binding of the assembly descriptor of an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeInterceptorBinding(final Element assemblyDescriptorElement,
            final IAssemblyDescriptor assemblyDescriptor) {
        // Get interceptor-binding list
        NodeList interceptorBindingList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(), IInterceptorBinding.NAME);

        // Loop on interceptor-binding
        for (int i = 0; i < interceptorBindingList.getLength(); i++) {
            Element interceptorBindingElement = (Element) interceptorBindingList.item(i);

            // Build struct and set it in the assembly descriptor
            InterceptorBinding interceptorBinding = new InterceptorBinding();
            assemblyDescriptor.addInterceptorBinding(interceptorBinding);

            // name
            String ejbName = XMLUtils.getStringValueElement(getXMLNS(), interceptorBindingElement, "ejb-name");
            interceptorBinding.setEjbName(ejbName);

            // interceptor-order
            NodeList interceptorOrderList = interceptorBindingElement.getElementsByTagNameNS(getXMLNS(), "interceptor-order");
            if (interceptorOrderList != null && interceptorOrderList.getLength() == 1) {
                // first item is the interceptor-order element
                Element interceptorOrderElement = (Element) interceptorOrderList.item(0);
                List<String> interceptorOrderClassList = XMLUtils.getStringListValueElement(getXMLNS(), interceptorOrderElement,
                        "interceptor-class");
                for (String interceptorClass : interceptorOrderClassList) {
                    interceptorBinding.addOrderedInterceptorClass(interceptorClass);
                }
            } else {
                // interceptor-class (as there is no interceptor-order element)
                List<String> interceptorClassList = XMLUtils.getStringListValueElement(getXMLNS(), interceptorBindingElement,
                        "interceptor-class");
                for (String interceptorClass : interceptorClassList) {
                    interceptorBinding.addInterceptorClass(interceptorClass);
                }
            }

            // method
            NodeList methodList = interceptorBindingElement.getElementsByTagNameNS(getXMLNS(), "method");
            if (methodList.getLength() > 0) {
                interceptorBinding.setMethod(getMethodFromElement((Element) methodList.item(0)));
            }

            // exclude-class-interceptors
            String excludeClassInterceptorsStr = XMLUtils.getStringValueElement(getXMLNS(), interceptorBindingElement,
                    "exclude-class-interceptors");
            if (excludeClassInterceptorsStr != null) {
                // convert into boolean value
                boolean excludeClassInterceptors = false;
                excludeClassInterceptors = Boolean.parseBoolean(excludeClassInterceptorsStr);
                interceptorBinding.setExcludeClassInterceptors(excludeClassInterceptors);
            }

            // exclude-default-interceptors
            String excludeDefaultInterceptorsStr = XMLUtils.getStringValueElement(getXMLNS(), interceptorBindingElement,
                    "exclude-default-interceptors");
            if (excludeDefaultInterceptorsStr != null) {
                // convert into boolean value
                boolean excludeDefaultInterceptors = false;
                excludeDefaultInterceptors = Boolean.parseBoolean(excludeDefaultInterceptorsStr);
                interceptorBinding.setExcludeDefaultInterceptors(excludeDefaultInterceptors);
            }

        }
    }

    /**
     * Analyze the application Exception of the assembly descriptor of an
     * EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeApplicationException(final Element assemblyDescriptorElement,
            final IAssemblyDescriptor assemblyDescriptor) {
        // Get application-exception list
        NodeList applicationExceptionList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(),
                JApplicationException.NAME);

        // Loop on application-exception
        for (int i = 0; i < applicationExceptionList.getLength(); i++) {
            Element applicationExceptionElement = (Element) applicationExceptionList.item(i);

            // name
            String className = XMLUtils.getStringValueElement(getXMLNS(), applicationExceptionElement, "exception-class");

            // rollback
            String rollbackStr = XMLUtils.getStringValueElement(getXMLNS(), applicationExceptionElement, "rollback");

            // convert into boolean value
            boolean rollback = false;
            if (rollbackStr != null) {
                rollback = Boolean.parseBoolean(rollbackStr);
            }

            // Build instance that is created.
            JApplicationException applicationException = new JApplicationException(className, rollback);
            assemblyDescriptor.addApplicationException(applicationException);
        }
    }

    /**
     * Analyze the container transaction element of the assembly descriptor of
     * an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeContainerTransaction(final Element assemblyDescriptorElement,
            final IAssemblyDescriptor assemblyDescriptor) {

        // Get container-transaction list
        NodeList containerTransactionList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(),
                IContainerTransaction.NAME);

        // Loop on container-transaction
        for (int i = 0; i < containerTransactionList.getLength(); i++) {
            Element containerTransactionElement = (Element) containerTransactionList.item(i);

            // Build struct and set it in the assembly descriptor
            ContainerTransaction containerTransaction = new ContainerTransaction();
            assemblyDescriptor.addContainerTransaction(containerTransaction);

            // Get Method element
            NodeList methodList = containerTransactionElement.getElementsByTagNameNS(getXMLNS(), "method");
            for (int j = 0; j < methodList.getLength(); j++) {
                Element methodElement = (Element) methodList.item(j);

                // Set method
                containerTransaction.addMethod(getMethodFromElement(methodElement));
            }

            // Tx attribute
            String transactionAttribute = XMLUtils.getStringValueElement(getXMLNS(), containerTransactionElement,
                    "trans-attribute");
            if (transactionAttribute != null) {
                if ("NotSupported".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(TransactionAttributeType.NOT_SUPPORTED);
                } else if ("Supports".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(TransactionAttributeType.SUPPORTS);
                } else if ("Required".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(TransactionAttributeType.REQUIRED);
                } else if ("RequiresNew".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(TransactionAttributeType.REQUIRES_NEW);
                } else if ("Mandatory".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(TransactionAttributeType.MANDATORY);
                } else if ("Never".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(TransactionAttributeType.NEVER);
                } else {
                    throw new IllegalStateException("Invalid transaction attribute '" + transactionAttribute + "'");
                }
            }
        }
    }

    /**
     * Analyze the security includind security-role, method-permission and
     * exclude-list elements of the assembly descriptor of an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeSecurity(final Element assemblyDescriptorElement, final IAssemblyDescriptor assemblyDescriptor) {

        // Get security-role list
        NodeList securityRoleList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(), "security-role");

        // Loop on security-role
        for (int i = 0; i < securityRoleList.getLength(); i++) {
            Element securityRoleElement = (Element) securityRoleList.item(i);

            // Get the security role name
            String securityRole = XMLUtils.getStringValueElement(getXMLNS(), securityRoleElement, "role-name");
            if (securityRole != null) {
                assemblyDescriptor.addSecurityRole(securityRole);
            }
        }

        // Get method-permission
        NodeList methodPermissionList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(), IMethodPermission.NAME);

        // Loop on method-permission
        for (int i = 0; i < methodPermissionList.getLength(); i++) {
            Element methodPermissionElement = (Element) methodPermissionList.item(i);

            // Build object
            MethodPermission methodPermission = new MethodPermission();
            assemblyDescriptor.addMethodPermission(methodPermission);

            // Get all the roles for this method permission
            List<String> roles = XMLUtils.getStringListValueElement(getXMLNS(), methodPermissionElement, "role-name");
            for (String roleName : roles) {
                methodPermission.addRoleName(roleName);
            }

            // Unchecked ?
            NodeList uncheckedList = methodPermissionElement.getElementsByTagNameNS(getXMLNS(), "unchecked");
            if (uncheckedList.getLength() > 0) {
                methodPermission.setUnchecked(true);
            }

            // Get Method element
            NodeList methodList = methodPermissionElement.getElementsByTagNameNS(getXMLNS(), "method");
            for (int j = 0; j < methodList.getLength(); j++) {
                Element methodElement = (Element) methodList.item(j);

                // Add method
                methodPermission.addMethod(getMethodFromElement(methodElement));
            }
        }

        // Get exclude-list
        NodeList excludeList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(), "exclude-list");

        // Loop on exclude-list (in fact only one element may be found on this
        // list)
        for (int i = 0; i < excludeList.getLength(); i++) {
            Element excludeListElement = (Element) excludeList.item(i);

            // Get Method element
            NodeList methodList = excludeListElement.getElementsByTagNameNS(getXMLNS(), "method");
            for (int j = 0; j < methodList.getLength(); j++) {
                Element methodElement = (Element) methodList.item(j);

                // Add method
                assemblyDescriptor.addExcludeList(getMethodFromElement(methodElement));
            }
        }

    }

    /**
     * Analyze the message-destination of the assembly descriptor of an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeMessageDestination(final Element assemblyDescriptorElement,
            final IAssemblyDescriptor assemblyDescriptor) {

        // Get message-destination list
        NodeList messageDestinationList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(), IMessageDestination.NAME);

        // Loop on message-destination
        for (int i = 0; i < messageDestinationList.getLength(); i++) {
            Element messageDestinationElement = (Element) messageDestinationList.item(i);

            // Create object
            IMessageDestination messageDestination = new MessageDestination();

            // Get the destination name
            String messageDestinationName = XMLUtils.getStringValueElement(getXMLNS(), messageDestinationElement,
                    "message-destination-name");
            if (messageDestinationName != null) {
                messageDestination.setName(messageDestinationName);
                assemblyDescriptor.addMessageDestination(messageDestination);
            }

            // Get the mappedName
            String mappedName = XMLUtils.getStringValueElement(getXMLNS(), messageDestinationElement, "mapped-name");
            if (mappedName != null) {
                messageDestination.setMappedName(mappedName);
            }


        }

    }

    /**
     * Analyze the lifecycle elements.
     * @param envElement the dom element.
     * @param env the structure representing object.
     */
    private void analyzeAroundInvoke(final Element envElement, final IEnvironment env) {
        // Get the around-invoke elements
        NodeList aroundInvokeList = envElement.getElementsByTagNameNS(getXMLNS(), "around-invoke");

        // Loop on the around-invoke
        for (int i = 0; i < aroundInvokeList.getLength(); i++) {
            Element aroundInvokeElement = (Element) aroundInvokeList.item(i);

            // Build instance that is created.
            AroundInvoke aroundInvoke = new AroundInvoke();
            env.addAroundInvoke(aroundInvoke);

            // lifecycle-callback-class
            String className = XMLUtils.getStringValueElement(getXMLNS(), aroundInvokeElement, "class");
            aroundInvoke.setClassName(className);

            // lifecycle-callback-method
            String method = XMLUtils.getStringValueElement(getXMLNS(), aroundInvokeElement, "method-name");
            aroundInvoke.setMethodName(method);

        }

    }

    /**
     * Add analyze of passivate interceptors and aroundInvoke.
     * @param envElement the xml environment element
     * @param env the environment element
     */
    @Override
    protected void analyzeEnvironment(final Element envElement, final IEnvironment env) {
        analyzeLifeCycle(envElement, env, PRE_PASSIVATE);
        analyzeLifeCycle(envElement, env, POST_ACTIVATE);
        analyzeAroundInvoke(envElement, env);
        super.analyzeEnvironment(envElement, env);
    }

    /**
     * @return the object resulting of the XML parsing.
     */
    public EJB3 getEJB3() {
        return ejb3;
    }

}
